/* 
 *  $Id: ChannelThread.java,v 1.16 1997/11/03 18:13:36 djw Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants;


/**
 * Thread of control that shepards a capsule from channel
 * receive through the node and onto an application.
 *
 * @author David Wetherall
 */


public class ChannelThread
  extends Thread
{

  // channel on behalf of which we live
  // set and fixed in constructor
  Channel source;

  // placeholder for soft-state generation identifier
  int cluster;

  public void run() {
    Node engine = source.client;
    Capsule c = null;

    while (true) {
      try {


	// need to clear this each loop in case receive throws
	c = null;

	// block while we receive and decode from the channel
	c = source.receive();

	// make it sequential, assume no recursion
	synchronized (engine) {
	  engine.context = c;
	  c.evaluate(engine);
	}

	if (c.saveBuffer) 
	  source.rxbuf = new Xdr(source.rxbuf.length());
      } catch (Exception e) {
	// log a problem but continue unheeded
	if (Entity.LOGGING)
	  source.log(Entity.L[0], "terminated " + c + "with " +e);
	e.printStackTrace();
      }
    }      
  }
  

  public ChannelThread(Channel c) {
    super();
    source = c;
  }
}


